==> PloneWeatherForecast-1.2.tgz: PloneWeatherForecast/PloneWeatherForecastTool.py <==
## -*- coding: utf-8 -*-
## Copyright (C)2006 Ingeniweb

## This program is free software; you can redistribute it and/or modify
## it under the terms of the GNU General Public License as published by
## the Free Software Foundation; either version 2 of the License, or
## (at your option) any later version.

## This program is distributed in the hope that it will be useful,
## but WITHOUT ANY WARRANTY; without even the implied warranty of
## MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
## GNU General Public License for more details.

## You should have received a copy of the GNU General Public License
## along with this program; see the file COPYING. If not, write to the
## Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

### This file is part of the Plone Weather software.
###
### (C) 2006 Emmanuel Coirier
###
### Plone Weather is a free software released under the GNU
### General Public License v2 as published by the Free Software Foundation.
###
### Please read and understand the LICENSE file placed in the doc directory.
###

import time
import re
import os


from AccessControl import ClassSecurityInfo
from AccessControl import getSecurityManager
from Globals import InitializeClass
from OFS.SimpleItem import SimpleItem
from OFS.PropertyManager import PropertyManager
from Products.PageTemplates.PageTemplateFile import PageTemplateFile

from Products.CMFCore import CMFCorePermissions
from Products.CMFCore.utils import UniqueObject

import Weather

# security.declareProtected(Permission, methodNameAsString)
# security.declarePrivate(methodNameAsString)
# security.declarePublic(methodNameAsString)
# default manager permission is CMFCorePermissions.ManagePortal

class PloneWeatherForecastTool(PropertyManager, UniqueObject, SimpleItem) :
    """
        Enable Plone to ask a weather forecast server.
    """
    id = 'weather_forecast_tool'
    meta_type = 'Plone Weather Forecast Tool'
    plone_tool = 1
    title = "Plone Weather Forecast Tool"

    _properties = (
        {'id':'title', 'type': 'string', 'mode':'w'},
        {'id':'id', 'type': 'string', 'mode':'w'},
        {'id':'key', 'type': 'string', 'mode':'w'},
        {'id':'location', 'type': 'string', 'mode':'w'},
        {'id':'dayForecast', 'type': 'int', 'mode':'w'},
        {'id':'dayForecastInPortlet', 'type': 'int', 'mode':'w'},
    )

    overviewPath = os.path.join(os.path.dirname(__file__), 'zmi', 'overview')

    manage_options = (

        ({'label' : 'Overview',
          'action' : 'overview'
          },)

        + PropertyManager.manage_options
        + SimpleItem.manage_options
    )

    security = ClassSecurityInfo()
    overview = PageTemplateFile(overviewPath)
    security.declareProtected(CMFCorePermissions.ManagePortal, 'overview')

    security.declarePrivate('manage_afterAdd')
    def manage_afterAdd(self, item, container) :
        """
            Create default structures
        """
        SimpleItem.manage_afterAdd(self, item, container)

        self.id = ""
        self.key = ""
        self.location = ""
        ## update every two hours (conforms to §8.2 of the weather.com's Weather XML Data Feed Implementation Guide)
        self.cacheDuration = 60 * 60 * 2
        self.dayForecast = 2
        self.dayForecastInPortlet = 2

        self.cache = {
            "date" : 0,
        }
        ## location is the google searchable location (ie Paris, France)
        ## lat and long are latitude and longitude (real values are allowed, as text representing real values are)
        ## spn is the zoom factor and is not parametrable
        self.mapUrl = "http://maps.google.com/maps?q=%(location)s&t=h&ll=%(lat)s,%(long)s&spn=0.003,0.003"

    security.declareProtected(CMFCorePermissions.ManagePortal, "manageFormResults")
    def manageFormResults(self, **params) :
        """
            Update the conf values
            returns a tuple with the status message and the next state
        """

        purgeCache = False
        if (self.id != params["id"]) or (self.key != params["key"]):
            purgeCache = True

        self.id = params["id"]
        self.key = params["key"]



        if self.location != params["location"] :
            ## weatherLocation has been changed by user
            try :
                Weather.checkLocation(params["location"], self.id, self.key)
                self.location = params["location"]
                purgeCache = True
            except RuntimeError, e :
                return ("Error : " + str(e), "error")

        if purgeCache and hasattr(self, "cache") :
            self.cache["date"] = 0
            self.cache = self.cache

        return ("pwf_config_ok", "success")

    security.declareProtected(CMFCorePermissions.ManagePortal, "flushCache")
    def flushCache(self):
        """
            handy method to flush cache
        """
        self.cache["date"] = 0
        self.cache = self.cache
        return "Cache flushed."

    security.declareProtected(CMFCorePermissions.ManagePortal, "searchLocation")
    def searchLocation(self, location):
        """
            Search for exact location given a town name.
            Return a tuple list with the code of the town and its exact name

            Note that since this method will be call for every call of the config
            template, passing None as the location will return None.
        """
        if location is None :
            return None
        return Weather.searchCity(location)

    security.declareProtected(CMFCorePermissions.ManagePortal, "searchLocation")
    def checkLocationFormat(self, location):
        """
            Checks that the location code is syntaxically correct
        """
        return re.match("[a-zA-Z]{4}\d{4}", location)

    security.declarePublic("getDayOfWeek")
    def getDayOfWeek(self, date):
        """
            Analyze a date to get the day of week.
            date is in the format :
            "6/7/06 3:00 PM Local Time"
            or
            "6/30/06 3:00 PM Local Time"
            for the iso date 2006-06-30...
        """
        ## Here, we remove everything this is after the third block
        ## (ie timezone value) since we are not sure strptime will
        ## correctly parsed it.
        ## new value would be "6/30/06 3:00 PM"
        date = " ".join(date.split(" ")[:3])
        parsedDate = time.strptime(date, "%m/%d/%y %I:%M %p")
        return time.strftime("%a", parsedDate)

    security.declarePublic("getShortNameFromLong")
    def getShortNameFromLong(self, longName):
        """
            Cut some characters from a day's name.
        """
        return longName[:3]

    security.declarePublic("getDayForecast")
    def getDayForecast(self, data):
        """
            Only return data for the day part of the day/night couple
            data is only one the days returned in the dayf/days part array.
        """
        for part in data["parts"]:
            if part["day_part"] == u"d":
                return part

        raise ValueException("No day part")

    security.declarePublic("getSimpleWeatherData")
    def getSimpleWeatherData(self):
        """
            Return a dictionnary with the following keys :
            location : the location returned by the server, or an error message.
            forecast : a list of forecast which contains for each element
                iconUrl : the url of the icon to display
                iconAlternativeText : the text of the icon
                temperatures : the temperature range, pretty formated
                dayOfWeek : a little label showing the day of week
        """
        data = self.getWeatherData()

        weather = {
            "location" : data["location"]["name"],
            "forecast" : []
        }

        weather["forecast"].append(
            ## current condition
            {
                "iconUrl" : "weather_icons/%s.png" % data["cc"]["icon_code"],
                "iconAlternativeText" : data["cc"]["text_description"],
                "temperatures": u"%s&nbsp;°%s" % (
                    data["cc"]["temperature"],
                    data["head"]["units"]["temperature"],
                ),
                "dayOfWeek": self.getDayOfWeek(data["cc"]["last_update"])
            }
        )

        for i in range(
            1,
            min(
                self.dayForecastInPortlet + 1,
                len(data["dayf"]["days"])
            )
        ):
            ## 0 is today, and we want forecasts for the day after, so we begin
            ## at 1.
            dData = self.getDayForecast(data["dayf"]["days"][i])
            w = {
                "iconUrl": "weather_icons/%s.png" % dData["icon_code"],
                "iconAlternativeText": dData["text_description"],
                "temperatures": u"%s&nbsp;°%s&nbsp;/&nbsp;%s&nbsp;°%s" % (
                    data["dayf"]["days"][i]["low_temperature"],
                    data["head"]["units"]["temperature"],
                    data["dayf"]["days"][i]["high_temperature"],
                    data["head"]["units"]["temperature"],
                ),
                "dayOfWeek": self.getShortNameFromLong(
                    data["dayf"]["days"][i]["day_text"]
                )
            }
            weather["forecast"].append(w)
        return weather

    security.declarePublic("manageMapUrl")
    def manageMapUrl(self, data):
        locationNameInUrl = data["location"]["name"].replace(" ", "+")
        data["location"]["mapUrl"] = self.mapUrl % {
            "location" : locationNameInUrl,
            "lat" : data["location"]["latitude"],
            "long" : data["location"]["longitude"],
        }
        return data

    security.declarePublic("getWeatherData")
    def getWeatherData(self) :
        """
            Return the complete dictionnary returned by Weather.py.
            More details can be found in the service DTD found at
            http://www.weather.com/documentation/xml/weather.dtd

            Run Weather.py if you want a sample dictionnary.
        """

        if self.cache["date"] + self.cacheDuration < time.time():
            ## cache has expired
            try:
                data = Weather.getWeather(
                    self.location,
                    self.id,
                    self.key,
                    days=max(self.dayForecastInPortlet + 1, self.dayForecast)
                )
                data = self.manageMapUrl(data)
                ## + 1 because 0 is the forecast for today and so is not what
                ## expected when talking about forecast.

                self.cache["data"] = data
                self.cache["date"] = time.time()
                self.cache = self.cache

            except RuntimeError, e:
                return {
                    "error" : e,
                    "iconUrl" : "weather_icons/na.png",
                    "iconAlternativeText" : "Forecast is not available.",
                }

        else :
            data = self.cache["data"]
        return data

    security.declarePublic("adminLink")
    def adminLink(self):
        """
            True if admin link have to be displayed
        """
        return getSecurityManager().checkPermission('Manager', self)



InitializeClass(PloneWeatherForecastTool)

==> PloneWeatherForecast-1.3.tgz: PloneWeatherForecast/PloneWeatherForecastTool.py <==
## -*- coding: utf-8 -*-
## Copyright (C)2006 Ingeniweb

## This program is free software; you can redistribute it and/or modify
## it under the terms of the GNU General Public License as published by
## the Free Software Foundation; either version 2 of the License, or
## (at your option) any later version.

## This program is distributed in the hope that it will be useful,
## but WITHOUT ANY WARRANTY; without even the implied warranty of
## MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
## GNU General Public License for more details.

## You should have received a copy of the GNU General Public License
## along with this program; see the file COPYING. If not, write to the
## Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

### This file is part of the Plone Weather software.
###
### (C) 2006 Emmanuel Coirier
###
### Plone Weather is a free software released under the GNU
### General Public License v2 as published by the Free Software Foundation.
###
### Please read and understand the LICENSE file placed in the doc directory.
###

import time
import re
import os


from AccessControl import ClassSecurityInfo
from AccessControl import getSecurityManager
from Globals import InitializeClass
from OFS.SimpleItem import SimpleItem
from OFS.PropertyManager import PropertyManager
from Products.PageTemplates.PageTemplateFile import PageTemplateFile

from Products.CMFCore import CMFCorePermissions
from Products.CMFCore.utils import UniqueObject

import Weather

# security.declareProtected(Permission, methodNameAsString)
# security.declarePrivate(methodNameAsString)
# security.declarePublic(methodNameAsString)
# default manager permission is CMFCorePermissions.ManagePortal

class PloneWeatherForecastTool(PropertyManager, UniqueObject, SimpleItem) :
    """
        Enable Plone to ask a weather forecast server.
    """
    id = 'weather_forecast_tool'
    meta_type = 'Plone Weather Forecast Tool'
    plone_tool = 1
    title = "Plone Weather Forecast Tool"

    _properties = (
        {'id':'title', 'type': 'string', 'mode':'w'},
        {'id':'id', 'type': 'string', 'mode':'w'},
        {'id':'key', 'type': 'string', 'mode':'w'},
        {'id':'location', 'type': 'string', 'mode':'w'},
        {'id':'dayForecast', 'type': 'int', 'mode':'w'},
        {'id':'dayForecastInPortlet', 'type': 'int', 'mode':'w'},
        {'id':'units', 'type': 'string', 'mode':'w'},
    )

    overviewPath = os.path.join(os.path.dirname(__file__), 'zmi', 'overview')

    manage_options = (

        ({'label' : 'Overview',
          'action' : 'overview'
          },)

        + PropertyManager.manage_options
        + SimpleItem.manage_options
    )

    security = ClassSecurityInfo()
    overview = PageTemplateFile(overviewPath)
    security.declareProtected(CMFCorePermissions.ManagePortal, 'overview')

    security.declarePrivate('manage_afterAdd')
    def manage_afterAdd(self, item, container) :
        """
            Create default structures
        """
        SimpleItem.manage_afterAdd(self, item, container)

        self.id = ""
        self.key = ""
        self.location = ""
        self.units = "m"
        ## update every two hours (conforms to §8.2 of the weather.com's Weather XML Data Feed Implementation Guide)
        self.cacheDuration = 60 * 60 * 2
        self.dayForecast = 2
        self.dayForecastInPortlet = 2

        self.cache = {
            "date" : 0,
        }
        ## location is the google searchable location (ie Paris, France)
        ## lat and long are latitude and longitude (real values are allowed, as text representing real values are)
        ## spn is the zoom factor and is not parametrable
        self.mapUrl = "http://maps.google.com/maps?q=%(location)s&t=h&ll=%(lat)s,%(long)s&spn=0.003,0.003"

    security.declareProtected(CMFCorePermissions.ManagePortal, "manageFormResults")
    def manageFormResults(self, **params) :
        """
            Update the conf values
            returns a tuple with the status message and the next state
        """

        purgeCache = False
        if (self.id != params["id"]) or (self.key != params["key"]):
            purgeCache = True

        self.id = params["id"]
        self.key = params["key"]

        if self.units != params["units"]:
            purgeCache = True
            self.units = params["units"]

        if self.location != params["location"] :
            ## weatherLocation has been changed by user
            try :
                Weather.checkLocation(params["location"], self.id, self.key)
                self.location = params["location"]
                purgeCache = True
            except RuntimeError, e :
                return ("Error : " + str(e), "error")

        if purgeCache and hasattr(self, "cache") :
            self.cache["date"] = 0
            self.cache = self.cache

        return ("pwf_config_ok", "success")

    security.declareProtected(CMFCorePermissions.ManagePortal, "flushCache")
    def flushCache(self):
        """
            handy method to flush cache
        """
        self.cache["date"] = 0
        self.cache = self.cache
        return "Cache flushed."

    security.declareProtected(CMFCorePermissions.ManagePortal, "searchLocation")
    def searchLocation(self, location):
        """
            Search for exact location given a town name.
            Return a tuple list with the code of the town and its exact name

            Note that since this method will be call for every call of the config
            template, passing None as the location will return None.
        """
        if location is None :
            return None
        return Weather.searchCity(location)

    security.declareProtected(CMFCorePermissions.ManagePortal, "searchLocation")
    def checkLocationFormat(self, location):
        """
            Checks that the location code is syntaxically correct
        """
        return re.match("[a-zA-Z]{4}\d{4}", location)

    security.declarePublic("getDayOfWeek")
    def getDayOfWeek(self, date):
        """
            Analyze a date to get the day of week.
            date is in the format :
            "6/7/06 3:00 PM Local Time"
            or
            "6/30/06 3:00 PM Local Time"
            for the iso date 2006-06-30...
        """
        ## Here, we remove everything this is after the third block
        ## (ie timezone value) since we are not sure strptime will
        ## correctly parsed it.
        ## new value would be "6/30/06 3:00 PM"
        date = " ".join(date.split(" ")[:3])
        parsedDate = time.strptime(date, "%m/%d/%y %I:%M %p")
        return time.strftime("%a", parsedDate)

    security.declarePublic("getShortNameFromLong")
    def getShortNameFromLong(self, longName):
        """
            Cut some characters from a day's name.
        """
        return longName[:3]

    security.declarePublic("getDayForecast")
    def getDayForecast(self, data):
        """
            Only return data for the day part of the day/night couple
            data is only one the days returned in the dayf/days part array.
        """
        for part in data["parts"]:
            if part["day_part"] == u"d":
                return part

        raise ValueError("No day part")

    security.declarePublic("getSimpleWeatherData")
    def getSimpleWeatherData(self):
        """
            Return a dictionnary with the following keys :
            location : the location returned by the server, or an error message.
            forecast : a list of forecast which contains for each element
                iconUrl : the url of the icon to display
                iconAlternativeText : the text of the icon
                temperatures : the temperature range, pretty formated
                dayOfWeek : a little label showing the day of week
        """
        data = self.getWeatherData()

        weather = {
            "location" : data["location"]["name"],
            "forecast" : []
        }

        weather["forecast"].append(
            ## current condition
            {
                "iconUrl" : "weather_icons/%s.png" % data["cc"]["icon_code"],
                "iconAlternativeText" : data["cc"]["text_description"],
                "temperatures": u"%s&nbsp;°%s" % (
                    data["cc"]["temperature"],
                    data["head"]["units"]["temperature"],
                ),
                "dayOfWeek": self.getDayOfWeek(data["cc"]["last_update"])
            }
        )

        for i in range(
            1,
            min(
                self.dayForecastInPortlet + 1,
                len(data["dayf"]["days"])
            )
        ):
            ## 0 is today, and we want forecasts for the day after, so we begin
            ## at 1.
            dData = self.getDayForecast(data["dayf"]["days"][i])
            w = {
                "iconUrl": "weather_icons/%s.png" % dData["icon_code"],
                "iconAlternativeText": dData["text_description"],
                "temperatures": u"%s&nbsp;°%s&nbsp;/&nbsp;%s&nbsp;°%s" % (
                    data["dayf"]["days"][i]["low_temperature"],
                    data["head"]["units"]["temperature"],
                    data["dayf"]["days"][i]["high_temperature"],
                    data["head"]["units"]["temperature"],
                ),
                "dayOfWeek": self.getShortNameFromLong(
                    data["dayf"]["days"][i]["day_text"]
                )
            }
            weather["forecast"].append(w)
        return weather

    security.declarePublic("manageMapUrl")
    def manageMapUrl(self, data):
        locationNameInUrl = data["location"]["name"].replace(" ", "+")
        data["location"]["mapUrl"] = self.mapUrl % {
            "location" : locationNameInUrl,
            "lat" : data["location"]["latitude"],
            "long" : data["location"]["longitude"],
        }
        return data

    security.declarePublic("getWeatherData")
    def getWeatherData(self) :
        """
            Return the complete dictionnary returned by Weather.py.
            More details can be found in the service DTD found at
            http://www.weather.com/documentation/xml/weather.dtd

            Run Weather.py if you want a sample dictionnary.
        """

        if self.cache["date"] + self.cacheDuration < time.time():
            ## cache has expired
            try:
                data = Weather.getWeather(
                    self.location,
                    self.id,
                    self.key,
                    days=max(self.dayForecastInPortlet + 1, self.dayForecast),
                    unit=self.units,
                )
                data = self.manageMapUrl(data)
                ## + 1 because 0 is the forecast for today and so is not what
                ## expected when talking about forecast.

                self.cache["data"] = data
                self.cache["date"] = time.time()
                self.cache = self.cache

            except RuntimeError, e:
                return {
                    "error" : e,
                    "iconUrl" : "weather_icons/na.png",
                    "iconAlternativeText" : "Forecast is not available.",
                }

        else :
            data = self.cache["data"]
        return data

    security.declareProtected(CMFCorePermissions.ManagePortal, "migrate")
    def migrate(self):
        """
            Check that all properties are present and correctly initialized.
            If not, install them with a safe default value.
            It is safe to call it multiple times.
        """
        checkFor = [
            ("units", "m"),
            ("dayForecast", 2),
            ("dayForecastInPortlet", 2)
        ]
        rValue = ""
        for attr, default in checkFor:
            if not hasattr(self, attr):
                setattr(self, attr, default)
                rValue += "Added: %s\n" % attr
        if rValue == "":
            return "Tool was already up to date."
        return rValue + "\nTool succesfully updated!"

    security.declarePublic("adminLink")
    def adminLink(self):
        """
            True if admin link have to be displayed
        """
        return getSecurityManager().checkPermission('Manager', self)



InitializeClass(PloneWeatherForecastTool)
